{
--------------------------------------------------------------------------------
  Vers. 3.0   J. Rathlev, March 2005
  TmMailer with extensions for multipart messaging and attachments with content types
  Added method "TAttachment.Clear";
  Function "Send" returns false on send error
  OnSendError reports e-mail address
  Setting of "ContentType" to attachments ("application/<subtype>" or "<type>/<subtype>")
    (Format of attachment strings: <filename>|<type> e.g. pdf,
     if <type> is omitted content-type has the default value "application/octet-stream")
  Messages with alternative text parts are supported
    (Set HTMLBody to HTML text, Body to plain text)

  Vers. 3.1
  Added ExtraHeaders
  in multipart messages plaintext is sent first (see RFC 1521 und 2046)

  Vers. 3.2
  works with Indy10    J. Rathlev, 2007-06-07

  Vers. 3.3
  port for SMTP added  J. Rathlev, 2007-11-21

  Vers. 3.4
  TLS support added J. Rathlev, 2008-08-27
    Requires:  Indy10-Versions of libeay32.dll, ssleay32.dll
               SSL certificate (created by OpenSSL)

  Vers. 3.5
  opt. Utf8 coding added J. Rathlev, 2009-03-05
  Content-Disposition: attachment

  Vers. 3.6
  CCList added J. Rathlev, 2010-05-07

  Vers. 3.7
  New error handling J. Rathlev, 2010-08-12
    On send failure look in "Errorlist"

  Vers. 3.8  J. Rathlev, 2013-06-06
  New events: OnBeginSend, OnSending, OnEndSend
  Enhanced error handling (IOHandler)
  User defined HeloName

  Vers. 4.0  J. Rathlev, 2014-02-28
  New: Authentication uses SASL mechanisms

  Vers. 4.1  J. Rathlev, 2014-06-30
  New: Optional logging of SMTP traffic for debugging purposes,
  Improved error handling

  Vers. 4.2  J. Rathlev, 2015-06-09
  New: Throw error if SSL is required and not available on send

  Vers. 4.3  J. Rathlev, 2017-01-16
  Changed: Loading of SSL libraries

  Vers. 4.4  J. Rathlev, 2017-05-24
  Added properties: Priority

  Vers. 4.5  J. Rathlev, 2019-03-25
  Added: SSLVersions:=[sslvTLSv1,sslvTLSv1_1,sslvTLSv1_2];

  Vers. 4.6  J. Rathlev, 2019-12-10
  Added: automatic setting of content type for attachments

  Vers. 4.7 J. Rathlev, 2022-03-31
  Changed: secure transfer definitions as type
  Fixed:   memory leaks on releasing SASLs

based on:
----------------------------------------------------------
MAS-CompMaker was used to generate this code
MAS-CompMaker, 2000-2002 MASoft, Mats Asplund
----------------------------------------------------------

Component Name: TmMailer
        Author: MASoft, Mats Asplund
      Creation: 2002-08-09
       Version: 2.1
   Description: A mailsender component
        Credit: (place your credits here)
        E-mail: mdp@telia.com
          Site: http://hem.passagen.se/matasp
  Legal issues: All rights reserved 2002 by MASoft, Mats Asplund

Usage:
  This software is provided 'as-is', without any express or
  implied warranty.  In no event will the author be held liable
  for any  damages arising from the use of this software.

  Permission is granted to anyone to use this software for any
  purpose, including commercial applications, and to alter it
  and redistribute it freely, subject to the following
  restrictions:

  1. The origin of this software must not be misrepresented,
     you must not claim that you wrote the original software.
     If you use this software in a product, an acknowledgment
     in the product documentation would be appreciated but is
     not required.

  2. Altered source versions must be plainly marked as such, and
     must not be misrepresented as being the original software.

  3. This notice may not be removed or altered from any source
     distribution.

  4. If you decide to use this software in any of your applications.
     Send me an EMail and tell me about it.

Quick Reference:
  TmMailer inherits from TComponent.

  Key-Properties:
    Body: The Body-part of the EMAil-message.
    Attachments
      AtttachedFiles: Add the files you wan't to attache to this StringList,
      FileName:       or just write filenames here.
    Host: Your Internet provider.
    UserId: Your UserId.
    Password: Your Password.
    Priority: Five levels of priority for the message.
    From: The sender EMail-address.
    Recipients: Add recipients to this list.
    CCList: Add recipient that get a carbon copy of the message
    BCCList: BccList is used to store email address of recipients that will
      receive a carbon copy of the message, but do not appear in the
      "To:" headers for the message. One way to prevent addresses to picked up
      by spammer.
    Subject: The messages subject.
    HTML: If true the EMail is marked as HTML else as plain text.
       (Rem. (J. Rathlev): This only works without attachements, see changes below)

  Key-Events:
    OnSendError: Event fired if there is an exception while sending.

  (New in version 2.1)
    OnConnected:    Event fired when client is connected.
    OnDisconnected: -------------"------------ disconnected.

  Key-Methods:
    Send: Sends the message.

--------------------------------------------------------------------------------
}
unit mMailer10;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.StdCtrls, IdGlobal, IdBaseComponent, IdMessage, IdComponent,
  IdTCPConnection, IdTCPClient, IdMessageClient, IdSMTP, IdEMailAddress, IdSMTPBase,
  IdHeaderList, IdAttachmentFile, IdExplicitTLSClientServerBase, IdIOHandler, IdText,
  IdUserPassProvider, IdSASLCollection, IdSASLUserPass, IdSASLPlain, IdLogFile,
  IdSASL_CRAM_SHA1, IdSASL_CRAMBase, IdSASL_CRAM_MD5, IdSASLLogin;

const
  sAbout = 'Version 4.7, 2005-2022  Mats Asplund, J. Rathlev';

  defHtmlType = 'text/html; charset=iso-8859-1';
  defTextType = 'text/plain; charset=iso-8859-1';
  defUtf8Type = 'text/plain; charset=utf-8';
  defEncoding = '8bit';
  defSmtpTimeOut = 30000;
  defAuthType = satSASL; //satDefault; //

  defLogName = 'SmtpLog.txt';

type
  TContentType = (ctAscii,ctHtml,ctUtf8);
  TSecTransfer = (stNone,stExplicit,stRequired,stImplicit);

  TOnSendError = procedure(Sender: TObject; ErrorMsg: string) of object;
  TGetPassword = procedure(var APassword : String) of object;

  TSendErrorType = (seUnknown,seSmtp,seFailedRecipient,seNoSecure);
  TSendError = record
    ErrType : TSendErrorType;
    ErrMsg  : string;
    end;
  TSendErrors = array of TSendError;

  TAttachement = class(TPersistent)
  private
    fAttachedFiles: TStrings;
    fOwner: TComponent;
    procedure SetAttachedFiles(const Value: TStrings);
  public
    constructor Create(Owner: TComponent);
    procedure Clear;
  published
    property AttachedFiles: TStrings read fAttachedFiles write SetAttachedFiles;
  end;

  TmMailer = class(TComponent)
  private
    fAbout: string;
    fHost: string;
    fPort : integer;
    fUserId: string;
    fFrom: TIdEMailAddressItem;
    fRecipients: TIdEMailAddressList;
    fCC: TIdEMailAddressList;
    fBCC: TIdEMailAddressList;
    fBody,fHTMLBody: TStrings;
    fSubject: string;
    fMsg: TIdMessage;
    fExtraHeaders: TIdHeaderList;
    fSMTP: TIdSMTP;
    fIOHandler: TIdIOHandler;
    fIdLogFile : TIdLogFile;
    fUserPassProvider: TIdUserPassProvider;
    fCertPwd,
    fPassword,
    fHeloName : string;
    fTLSAvail : Boolean;
    fContentType : TContentType;
    fPriority: TIdMessagePriority;
    fOnDisconnected: TNotifyEvent;
    fOnConnected: TNotifyEvent;
    FOnSending: TWorkEvent;
    FOnBeginSend: TWorkBeginEvent;
    FOnEndSend: TWorkEndEvent;
    fAttachments: TAttachement;
    fGetSSLPassword : TGetPassword;
    fUseTLS : TIdUseTLS;
    fContinueOnFailedRecipient : boolean;
    fErrorList : TSendErrors;
    procedure SetBody(const Value: TStrings);
    procedure SetHTMLBody(const Value: TStrings);
    procedure IsConnected(Sender: TObject);
    procedure Disconnected(Sender: TObject);
    procedure SetTLS(Value : TSecTransfer);
    procedure InitIOHandler;
    procedure GetCertPassword(var Password : String);
    function GetSize : int64;
    procedure DoBegin (ASender: TObject; AWorkMode: TWorkMode; AWorkCountMax: Int64);
    procedure DoSend (ASender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
    procedure DoEnd (ASender: TObject; AWorkMode: TWorkMode);
    procedure FailedRecipient (Sender: TObject; const AAddress, ACode, AText: String; var VContinue: Boolean);
    function GetLogName : string;
    procedure SetLogName (Value : string);
    procedure SetWriteLog (Value : boolean);
    procedure AddError(seType : TSendErrorType; const seMsg : string);
    procedure PrepareMessage;
  public
    constructor Create(AOwner: TComponent; const Path,Root,Cert,Key,Pwd : string); overload;
    constructor Create (AOwner : TComponent); overload;
    destructor Destroy; override;
    procedure ClearBody;
    procedure AddToBody (const AText : string);
    procedure ClearExtraHeaders;
    procedure AddExtraHeader (const fId,fContents : string);
    function SaveToFile (const AFilename : string) : boolean;
    function Send : boolean;
  published
    property About: string read fAbout;
    property Body: TStrings read fBody write SetBody;
    property HTMLBody: TStrings read fHTMLBody write SetHTMLBody;
    property Attachments: TAttachement read fAttachments write fAttachments;
    property Host: string read fHost write fHost;
    property Port: integer read fPort write fPort;
    property UserId: string read fUserId write fUserId;
    property Password: string read fPassword write fPassword;
    property HeloName : string read fHeloName write fHeloName;
    property Priority: TIdMessagePriority read fPriority write fPriority default ID_MSG_PRIORITY;
    property From: TIdEMailAddressItem read fFrom write fFrom;
    property Recipients: TIdEMailAddressList read fRecipients write fRecipients;
    property CCList: TIdEMailAddressList read fCC write fCC;
    property BCCList: TIdEMailAddressList read fBCC write fBCC;
    property Subject: string read fSubject write fSubject;
    property ContentType: TContentType read fContentType write fContentType;
    property SecureTransfer : TSecTransfer write SetTLS;
    property ContinueOnFailedRecipient : boolean read fContinueOnFailedRecipient write fContinueOnFailedRecipient;
    property ErrorList : TSendErrors read fErrorList;
    property Logname : string read GetLogName write SetLogName;
    property WriteLogFile : boolean write SetWriteLog;
    property OnConnected: TNotifyEvent read fOnConnected write fOnConnected;
    property OnDisconnected: TNotifyEvent read fOnDisconnected write fOnDisconnected;
    property OnSending: TWorkEvent read FOnSending write FOnSending;
    property OnBeginSend: TWorkBeginEvent read FOnBeginSend write FOnBeginSend;
    property OnEndSend: TWorkEndEvent read FOnEndSend write FOnEndSend;
    property OnSSLPassword: TGetPassword read fGetSSLPassword write fGetSSLPassword;
    end;

  TSendMailThread = class (TThread)
  private
    FMailer : TmMailer;
    FOnError : TNotifyEvent;
    procedure CallOnError;
  protected
    procedure Execute; override;
  public
    constructor Create (AMailer : TmMailer; AOnError : TNotifyEvent);
    property Terminated;
    end;

procedure Register;

implementation

uses IdAssignedNumbers, IdSSLOpenSSL, IdException, IdResourceStringsProtocols,
  IdSSLOpenSSLHeaders, IdGlobalProtocols, System.StrUtils, Vcl.Forms;

function TempDirectory : string;
var
  p : pchar;
begin
  p:=StrAlloc(MAX_PATH+1);
  GetTempPath(MAX_PATH+1,p);
  Result:=p;
  Strdispose(p);
  end;

{ ------------------------------------------------------------------- }
{ TAttachement }

constructor TAttachement.Create(Owner: TComponent);
begin
  inherited Create;
  fOwner:= Owner;
  end;

procedure TAttachement.SetAttachedFiles(const Value: TStrings);
begin
  fAttachedFiles.Assign(Value);
  end;

// added Mar. 2004, J. Rathlev
procedure TAttachement.Clear;
begin
  fAttachedFiles.Clear;
  end;

{ ------------------------------------------------------------------- }
{ TmMailer }

constructor TmMailer.Create(AOwner: TComponent; const Path,Root,Cert,Key,Pwd : string);
var
  sd,sr,sc,sk : string;
begin
  inherited Create(AOwner);
  fAttachments:= TAttachement.Create(self);
  fAttachments.fAttachedFiles:= TStringList.Create;
  fPort:=IdPORT_SMTP;
  fBody:= TStringList.Create;
  fHTMLBody:= TStringList.Create;
  fRecipients:= TIdEMailAddressList.Create(Self);
  fCC:= TIdEMailAddressList.Create(Self);
  fBCC:= TIdEMailAddressList.Create(Self);
  fFrom:= TIdEMailAddressItem.Create(nil);
  fMsg:= TIdMessage.Create(Self);
  fSMTP:= TIdSMTP.Create(Self);
  fHeloName:='Test';
  fUseTLS:=utNoTLSSupport;
  fUserPassProvider:=TIdUserPassProvider.Create(nil);
  with fSMTP do begin
    ConnectTimeout:=defSmtpTimeOut;
    ReadTimeout:=defSmtpTimeOut;
    fCertPwd:='';
    ManagedIOHandler:=True;
    fUseTLS:=utNoTLSSupport;
    HeloName:='Test';
  // prepare SASL mechanisms
    with SASLMechanisms do begin
      with Add do begin
        Sasl:=TIdSASLLogin.Create(nil);
        (Sasl as TIdSASLUserPass).UserPassProvider:=fUserPassProvider;
        end;
      with Add do begin
        Sasl:=TIdSASLPlain.Create(nil);
        (Sasl as TIdSASLUserPass).UserPassProvider:=fUserPassProvider;
        end;
      with Add do begin
        Sasl:=TIdSASLCRAMMD5.Create(nil);
        (Sasl as TIdSASLUserPass).UserPassProvider:=fUserPassProvider;
        end;
      with Add do begin
        Sasl:=TIdSASLCRAMSHA1.Create(nil);
        (Sasl as TIdSASLUserPass).UserPassProvider:=fUserPassProvider;
        end;
      end;
    OnConnected:=IsConnected;
    OnDisconnected:=Disconnected;
    OnFailedRecipient:=FailedRecipient;
    FOnBeginSend:=nil;
    FOnSending:=nil;
    FOnEndSend:=nil;
    OnWorkBegin:=DoBegin;
    OnWork:=DoSend;
    OnWorkEnd:=DoEnd;
    end;
// prepare secure connection
// if openSSL libraries and certificates are available, use SSL for TLS support
  fTLSAvail:=(length(Root)>0) and (length(Cert)>0) and (length(Key)>0);
  if fTLSAvail then begin
//    sd:=GetCurrentDir;
//    if not AnsiSametext(IncludeTrailingPathDelimiter(sd),Path) then begin
//      SetCurrentDir(Path);
    IdOpenSSLSetLibPath(Path); // search for ssleay32.dll and libeay32.dll in certificates path
    try
      TIdSSLContext.Create.Free;
      fTLSAvail:=true;
    except
      fTLSAvail:=false;
      end;
//      SetCurrentDir(sd);
    if not fTLSAVail then begin  // not found, search in current path
      IdOpenSSLSetLibPath('');
      try
        TIdSSLContext.Create.Free;
        fTLSAvail:=true;
      except
        fTLSAvail:=false;
        end;
      end;
    if fTLSAvail then begin
      sr:=Path+Root; sc:=Path+Cert; sk:=Path+Key;
      fTLSAvail:=FileExists(sr) and FileExists(sc) and FileExists(sk);
      end;
    end;
  if fTLSAvail then begin
    fIOHandler:=TIdSSLIOHandlerSocketOpenSSL.Create(fSmtp);
    fCertPwd:=Pwd;
    with (fIOHandler as TIdSSLIOHandlerSocketOpenSSL)do begin
      with SSLOptions do begin
        RootCertFile:=sr; CertFile:=sc; KeyFile:=sk;
        SSLVersions:=[sslvTLSv1,sslvTLSv1_1,sslvTLSv1_2];
        end;
      OnGetPassword:=GetCertPassword;
      end;
    end
  else fIOHandler:=TIdIOHandler.MakeDefaultIOHandler(fSmtp);
  fSMTP.IOHandler:=fIOHandler;
  fIdLogFile:=TIdLogFile.Create(nil);          //create the log file
  with fidLogFile do begin
    FileName:=IncludeTrailingPathDelimiter(TempDirectory)+defLogName;
    end;
  fIOHandler.Intercept:=fIdLogFile;
  fContentType:=ctAscii;
  fPriority:= mpNormal;
  fExtraHeaders:=TIdHeaderList.Create(QuoteMIME);
  fAbout:=sAbout;
  fOnConnected:=nil;
  fOnDisconnected:=nil;
  fGetSSLPassword:=nil;
  fContinueOnFailedRecipient:=false;
  fErrorList:=nil;
  end;

constructor TmMailer.Create (AOwner : TComponent);
begin
  Create(AOwner,'','','','','');
  end;

destructor TmMailer.Destroy;
var
  i : integer;
begin
  // release SASL mechanisms
  with fSMTP.SASLMechanisms do for i:=Count-1 downto 0 do Items[i].SASL.Free;
  fAttachments.fAttachedFiles.Free; fAttachments.Free;
  fBody.Free; fHTMLBody.Free; fMsg.Free;
  fRecipients.Free; fCC.Free; fBCC.Free; fFrom.Free;
  fUserPassProvider.Free;
  fExtraHeaders.Free;
  fIdLogFile.Free;
  fSMTP.IOHandler.Free;
  fSMTP.Free;
  inherited Destroy;
  end;

procedure TmMailer.InitIOHandler;
begin
  end;

procedure TmMailer.GetCertPassword(var Password : String);
begin
  if Assigned(fGetSSLPassword) then fGetSSLPassword(Password)  // prompt for password
  else Password:=fCertPwd;
  end;

function TmMailer.GetLogName : string;
begin
  Result:=fIdLogFile.FileName;
  end;

procedure TmMailer.SetLogName (Value : string);
begin
  fIdLogFile.FileName:=Value;
  end;

procedure TmMailer.SetWriteLog (Value : boolean);
begin
  fIdLogFile.Active:=Value;
  end;

procedure TmMailer.IsConnected(Sender: TObject);
begin
  if Assigned(fOnConnected) then fOnConnected(self);
  end;

procedure TmMailer.Disconnected(Sender: TObject);
begin
  if Assigned(fOnDisconnected) then fOnDisconnected(self);
  end;

procedure TmMailer.DoBegin (ASender: TObject; AWorkMode: TWorkMode; AWorkCountMax: Int64);
begin
  if assigned(FOnBeginSend) then FOnBeginSend(ASender,AWorkMode,GetSize);
  end;

procedure TmMailer.DoEnd (ASender: TObject; AWorkMode: TWorkMode);
begin
  if assigned(FOnEndSend) then FOnEndSend(ASender,AWorkMode);
  end;

procedure TmMailer.DoSend (ASender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if assigned(FOnSending) then FOnSending(ASender,AWorkMode,AWorkCount)
  else Application.ProcessMessages;
  end;

procedure TmMailer.AddError(seType : TSendErrorType; const seMsg : string);
var
  n : integer;
begin
  n:=length(fErrorList);
  SetLength(fErrorList,n+1);
  with fErrorList[n] do begin
    ErrType:=seType;
    ErrMsg:=seMsg;
    end;
  end;

procedure TmMailer.FailedRecipient(Sender: TObject; const AAddress, ACode, AText: String;
                                   var VContinue: Boolean);
var
  s : string;

  function DelCtrlChars(const S : string) : string;
  var
    i  : integer;
  begin
    Result:=s;
    if length(Result)>0 then begin
      i:=1;
      while (i<=length(Result)) do begin
        if Result[i]<#32 then delete(Result,i,1) else inc(i);
        end;
      end;
    end;

begin
  VContinue:=fContinueOnFailedRecipient;
  if length(AAddress)=0 then s:=DelCtrlChars(AText)  // remove cr/lf
  else s:=AAddress;
  AddError(seFailedRecipient,s);
  end;

procedure TmMailer.SetTLS(Value : TSecTransfer);
begin
  case Value of
  stExplicit : fUseTLS:=utUseExplicitTLS;
  stRequired : fUseTLS:=utUseRequireTLS;
  stImplicit : fUseTLS:=utUseImplicitTLS;
  else fUseTLS:=utNoTLSSupport;
    end;
  end;

procedure TmMailer.ClearExtraHeaders;
begin
  fExtraHeaders.Clear;
  end;

procedure TmMailer.ClearBody;
begin
  fBody.Clear;
  end;

// Replacement for Body.Add()
procedure TmMailer.AddToBody (const AText : string);
begin
//  if fContentType=ctUtf8 then fBody.Add(UTF8Encode(AText)) else
  fBody.Add(AText);
  end;

procedure TmMailer.AddExtraHeader (const fId,fContents : string);
begin
  fExtraHeaders.Values[fId]:=fContents;
  end;

function TmMailer.GetSize : int64;
var
  fs : TFileStream;
  tn : string;
begin
  tn:=TempDirectory+'SendMail.tmp';
  fs:=TFileStream.Create(tn,fmCreate);
  try fMsg.SaveToStream(fs); except end;
  Result:=fs.Size;
  fs.Free;
  System.SysUtils.DeleteFile(tn);
  end;

procedure TmMailer.PrepareMessage;
var
  n,k  : integer;
  s,sf : string;
  at   : TIdAttachmentFile;
begin
  with fMsg do begin        // inserted, J. Rathlev, Jan. 2004
    ClearHeader; ClearBody;
    end;
  // handling of multipart messages, J. Rathlev, Jun. 2007
  if fContentType=ctUtf8 then begin
    fSMTP.IOHandler.DefStringEncoding:=enUtf8;
    with TIdText.Create(fMsg.MessageParts,fBody) do begin
      ContentType:=defUtf8Type;
      ContentTransfer:=defEncoding;
      end;
    end
  else if (fContentType=ctHtml) and (fBody.Count>0) then begin
  // multipart/alternative if HTML, text and Attachemets
    with TIdText.Create(fMsg.MessageParts,fBody) do begin
      ContentType:='multipart/alternative';  // Container for text parts
      end;
    with TIdText.Create(fMsg.MessageParts,fBody) do begin
      ContentType:=defTextType;
      ContentTransfer:=defEncoding;
      ParentPart:=0;
      end;
    with TIdText.Create(fMsg.MessageParts,fHTMLBody) do begin
      ContentType:=defHtmlType;
      ContentTransfer:=defEncoding;
      ParentPart:=0;
      end;
    end
  else if fContentType=ctHtml then begin
    with TIdText.Create(fMsg.MessageParts,fHTMLBody) do begin
      ContentType:=defHtmlType;
      ContentTransfer:=defEncoding;
      end;
    end
  else begin
    with TIdText.Create(fMsg.MessageParts,fBody) do begin
      ContentType:=defTextType;
      ContentTransfer:=defEncoding;
      end;
    end;
  with fAttachments do if AttachedFiles.Count>0 then begin // message with attachements
    fMsg.ContentDisposition:='attachment';
    for n:= 0 to AttachedFiles.Count-1 do begin  // attachments
      s:=AttachedFiles[n];
      k:=pos('|',s);                       // separate filename and content type
      if k=0 then k:=length(s)+1;
      sf:=copy(s,1,k-1);
      at:=TIdAttachmentFile.Create(fMsg.MessageParts,sf);
      delete(s,1,k);
      if length(s)>0 then begin
        k:=pos('/',s);   // other than application?
        if k=0 then at.ContentType:='application/'+s
        else begin
          at.ContentType:=s;
          if AnsiStartsText('text/plain',s) then at.ContentTransfer:=defEncoding;
          end;
        end
      else begin
        s:=GetMIMETypeFromFile(ExtractFileName(sf));
        at.ContentType:=s;
        if AnsiStartsText('text/plain',s) then at.ContentTransfer:=defEncoding;
        end;
      end;
    end;
  with fMsg do begin
    Recipients:=fRecipients;
    CCList:=fCC;
    BccList:=fBCC;
    From:=fFrom;
    Subject:=fSubject;
    ExtraHeaders:=fExtraHeaders;
    Priority:=fPriority;
    end;
  end;

function TmMailer.SaveToFile (const AFilename : string) : boolean;
begin
  try
    PrepareMessage;
    fMsg.SaveToFile(AFilename);
    Result:=true;
  except
    Result:=false;
    end;
  end;

function TmMailer.Send : boolean;
var
  s  : string;

  procedure Authenticate;
  begin
    if not fSMTP.Authenticate then
      raise EIdException.Create(RSSMTPSvrAuthFailed);
    end;

  procedure WriteFailedLoadList(const AText : string);
  var
    sl : TStringList;
    i  : integer;
  begin
    sl:=TStringList.Create;
    sl.CommaText:=AText;
    with fSMTP.IOHandler.Intercept as TIdLogFile do begin
      LogWriteLine(RSGeneralError+DateTimeToStr(Now),RSSSLCreatingSessionError);
      for i:=0 to sl.Count-1 do LogError(Format(RSSSLLoadFunctionError,[sl[i]]));
      end;
    sl.Free;
    end;

begin
  Result:=true;
  fErrorList:=nil;
  // check if secure transfer is availbale
  if (fUseTLS<>utNoTLSSupport) and not fTLSAvail then begin
    AddError(seNoSecure,RSSSLCreatingSessionError);
    s:=WhichFailedToLoad;
    if (length(s)>0) and fIdLogFile.Active then WriteFailedLoadList(s);
    end
  else begin
    PrepareMessage;
    with fUserPassProvider do begin
      Username:=fUserId;
      Password:=fPassword;
      end;

    with fSMTP do begin
      if (length(fUserId)>0) and (length(fPassword)>0) then fSMTP.AuthType:=defAuthType // satDefault
      else fSMTP.AuthType:=satNone;
      UserName:=fUserId;
      Password:=fPassword;
      HeloName:=fHeloName;
      UseTLS:=fUseTLS;
      Host:= fHost;
      Port:= fPort;
      end;

    try
      fSMTP.Connect;
      try
        Authenticate;
        fSMTP.Send(fMsg);
      finally
        fSMTP.Disconnect(true);
        end;
    except
      on E:EIdException do begin
        AddError(seSmtp,E.Message);
        end
      else AddError(seUnknown,RSIdentUnknownError);
      end;
    end;
  Result:=length(fErrorList)=0;
  fSMTP.IOHandler:=fIOHandler;
  end;

procedure TmMailer.SetBody(const Value: TStrings);
begin
  fBody.Assign(Value);
  end;

procedure TmMailer.SetHTMLBody(const Value: TStrings);
begin
  fHTMLBody.Assign(Value);
  end;

{ ---------------------------------------------------------------- }
constructor TSendMailThread.Create (AMailer : TmMailer; AOnError : TNotifyEvent);
begin
  inherited Create(false);
  Priority:=tpLower;   // process priority
  FreeOnTerminate:=true;
  FMailer:=AMailer;
  FOnError:=AOnError;
  end;

procedure TSendMailThread.CallOnError;
begin
  if assigned(FOnError) then FOnError(self);
  end;

procedure TSendMailThread.Execute;
begin
  if not FMailer.Send and assigned(FOnError) then Synchronize(CallOnError);
  Terminate;
  end;

{ ---------------------------------------------------------------- }
procedure Register;
begin
  RegisterComponents('Internet', [TmMailer]);
  end;

end.

